#!/usr/bin/python
# Copyright 2009 Canonical Ltd.
#
# This file is part of desktopcouch.
#
#  desktopcouch is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License version 3
# as published by the Free Software Foundation.
#
# desktopcouch is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with desktopcouch.  If not, see <http://www.gnu.org/licenses/>.
#
# Author: Stuart Langridge <stuart.langridge@canonical.com>

"""CouchDB port advertiser.

A command-line utility which exports a
desktopCouch.getPort method on the bus which returns
that port, so other apps (specifically, the contacts API) can work out
where CouchDB is running so it can be talked to.

Calculates the port number by looking in the CouchDB log.

If CouchDB is not running, then run the script to start it and then
start advertising the port.

This file should be started by D-Bus activation.

"""

import local_files

from dbus.mainloop.glib import DBusGMainLoop
import dbus.service, gobject, os, errno, time
DBusGMainLoop(set_as_default=True)

class PortAdvertiser(dbus.service.Object):
    "Advertise the passed port number on the D-Bus Session bus"
    def __init__(self, port, death):
        # XXX: change bus name to not include ubuntuone
        bus_name = dbus.service.BusName("org.desktopcouch.CouchDB",
                     bus=dbus.SessionBus())
        self.port = int(port)
        self.death = death
        dbus.service.Object.__init__(self, object_path="/", bus_name=bus_name)

    @dbus.service.method(dbus_interface='org.desktopcouch.CouchDB',
                         in_signature='', out_signature='i')
    def getPort(self):
        "Exported method to return the port"
        return self.port

    @dbus.service.method(dbus_interface='org.desktopcouch.CouchDB',
                         in_signature='', out_signature='')
    def quit(self):
        "Exported method to quit the program"
        self.death()

if __name__ == "__main__":
    # Work out whether CouchDB is running by looking at its pid file
    fp = open(local_files.FILE_PID)
    pid = int(fp.read())
    fp.close()
    try:
        os.kill(pid, 0)
    except os.error, detail:
        if detail.errno == errno.ESRCH:
            # pidfile is stale
            # start CouchDB by running the startup script
            print "Desktop CouchDB is not running; starting it."
            import start_local_couchdb
            start_local_couchdb.start_couchdb()
            time.sleep(2) # give the process a chance to start

    # Look in the CouchDB log to find the port number
    # FIXME: currently do this by running the world's most complex lsof command
    import subprocess
    lsofcmd = 'lsof -i4TCP@localhost | grep $(couchdb -C %s -p %s -s | ' + \
              'cut -d" " -f7 | cut -d "," -f1) | grep "TCP " | ' + \
              'grep LISTEN | awk \'{print $8}\' | cut -d":" -f2'
    actual_lsof_cmd = lsofcmd % (local_files.FILE_INI, local_files.FILE_PID)
    stdout, stderr = subprocess.Popen(actual_lsof_cmd, shell=True,
      stdout=subprocess.PIPE).communicate()
    port = stdout.strip()

    # Advertise the port
    mainloop = gobject.MainLoop()
    portAdvertiser = PortAdvertiser(port, mainloop.quit)
    mainloop.run()
